#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Spatial and Geostatistics
- Name     : Kriging
- ID       : statistics_kriging

Description
----------
Kriging - tools for the geostatistical interpolation of irregularly distributed point data.
'''

from PySAGA.helper import Tool_Wrapper

def Ordinary_Kriging(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Ordinary Kriging
    ----------
    [statistics_kriging.0]\n
    Ordinary Kriging for grid interpolation from irregular sample points.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTION [`output grid`] : Prediction
    - VARIANCE [`output grid`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '0', 'Ordinary Kriging')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False

def run_tool_statistics_kriging_0(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Ordinary Kriging
    ----------
    [statistics_kriging.0]\n
    Ordinary Kriging for grid interpolation from irregular sample points.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTION [`output grid`] : Prediction
    - VARIANCE [`output grid`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '0', 'Ordinary Kriging')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False

def Simple_Kriging(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Simple Kriging
    ----------
    [statistics_kriging.1]\n
    Simple Kriging for grid interpolation from irregular sample points.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTION [`output grid`] : Prediction
    - VARIANCE [`output grid`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '1', 'Simple Kriging')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False

def run_tool_statistics_kriging_1(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Simple Kriging
    ----------
    [statistics_kriging.1]\n
    Simple Kriging for grid interpolation from irregular sample points.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTION [`output grid`] : Prediction
    - VARIANCE [`output grid`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '1', 'Simple Kriging')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False

def Universal_Kriging(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTORS=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None,
    RESAMPLING=None,
    COORDS=None, Verbose=2):
    '''
    Universal Kriging
    ----------
    [statistics_kriging.2]\n
    Universal Kriging for grid interpolation from irregular sample points.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTORS [`optional input grid list`] : Predictors
    - PREDICTION [`output grid`] : Prediction
    - VARIANCE [`output grid`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points
    - RESAMPLING [`choice`] : Resampling. Available Choices: [0] Nearest Neighbour [1] Bilinear Interpolation [2] Bicubic Spline Interpolation [3] B-Spline Interpolation Default: 3
    - COORDS [`boolean`] : Coordinates. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '2', 'Universal Kriging')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Input ('PREDICTORS', PREDICTORS)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        Tool.Set_Option('RESAMPLING', RESAMPLING)
        Tool.Set_Option('COORDS', COORDS)
        return Tool.Execute(Verbose)
    return False

def run_tool_statistics_kriging_2(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTORS=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None,
    RESAMPLING=None,
    COORDS=None, Verbose=2):
    '''
    Universal Kriging
    ----------
    [statistics_kriging.2]\n
    Universal Kriging for grid interpolation from irregular sample points.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTORS [`optional input grid list`] : Predictors
    - PREDICTION [`output grid`] : Prediction
    - VARIANCE [`output grid`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points
    - RESAMPLING [`choice`] : Resampling. Available Choices: [0] Nearest Neighbour [1] Bilinear Interpolation [2] Bicubic Spline Interpolation [3] B-Spline Interpolation Default: 3
    - COORDS [`boolean`] : Coordinates. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '2', 'Universal Kriging')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Input ('PREDICTORS', PREDICTORS)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        Tool.Set_Option('RESAMPLING', RESAMPLING)
        Tool.Set_Option('COORDS', COORDS)
        return Tool.Execute(Verbose)
    return False

def Regression_Kriging(
    POINTS=None,
    PREDICTORS=None,
    REGRESSION=None,
    PREDICTION=None,
    RESIDUALS=None,
    VARIANCE=None,
    INFO_COEFF=None,
    INFO_MODEL=None,
    INFO_STEPS=None,
    FIELD=None,
    TQUALITY=None,
    COORD_X=None,
    COORD_Y=None,
    INTERCEPT=None,
    METHOD=None,
    P_VALUE=None,
    RESAMPLING=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    KRIGING=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Regression Kriging
    ----------
    [statistics_kriging.3]\n
    Regression Kriging for grid interpolation from irregular sample points.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - PREDICTORS [`input grid list`] : Predictors
    - REGRESSION [`output grid`] : Regression. regression model applied to predictor grids
    - PREDICTION [`output grid`] : Prediction
    - RESIDUALS [`output grid`] : Residuals
    - VARIANCE [`output grid`] : Prediction Error
    - INFO_COEFF [`output table`] : Regression: Coefficients
    - INFO_MODEL [`output table`] : Regression: Model
    - INFO_STEPS [`output table`] : Regression: Steps
    - FIELD [`table field`] : Attribute
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] standard deviation [1] variance Default: 0
    - COORD_X [`boolean`] : Include X Coordinate. Default: 0
    - COORD_Y [`boolean`] : Include Y Coordinate. Default: 0
    - INTERCEPT [`boolean`] : Intercept. Default: 1
    - METHOD [`choice`] : Method. Available Choices: [0] include all [1] forward [2] backward [3] stepwise Default: 3
    - P_VALUE [`floating point number`] : Significance Level. Minimum: 0.000000 Maximum: 100.000000 Default: 5.000000 Significance level (aka p-value) as threshold for automated predictor selection, given as percentage
    - RESAMPLING [`choice`] : Resampling. Available Choices: [0] Nearest Neighbour [1] Bilinear Interpolation [2] Bicubic Spline Interpolation [3] B-Spline Interpolation Default: 3
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes for variogram estimation
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Variogram Model. Default: a + b * x
    - KRIGING [`choice`] : Kriging Type. Available Choices: [0] Simple Kriging [1] Ordinary Kriging Default: 0
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '3', 'Regression Kriging')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('PREDICTORS', PREDICTORS)
        Tool.Set_Output('REGRESSION', REGRESSION)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('RESIDUALS', RESIDUALS)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('INFO_COEFF', INFO_COEFF)
        Tool.Set_Output('INFO_MODEL', INFO_MODEL)
        Tool.Set_Output('INFO_STEPS', INFO_STEPS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('COORD_X', COORD_X)
        Tool.Set_Option('COORD_Y', COORD_Y)
        Tool.Set_Option('INTERCEPT', INTERCEPT)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('P_VALUE', P_VALUE)
        Tool.Set_Option('RESAMPLING', RESAMPLING)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('KRIGING', KRIGING)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False

def run_tool_statistics_kriging_3(
    POINTS=None,
    PREDICTORS=None,
    REGRESSION=None,
    PREDICTION=None,
    RESIDUALS=None,
    VARIANCE=None,
    INFO_COEFF=None,
    INFO_MODEL=None,
    INFO_STEPS=None,
    FIELD=None,
    TQUALITY=None,
    COORD_X=None,
    COORD_Y=None,
    INTERCEPT=None,
    METHOD=None,
    P_VALUE=None,
    RESAMPLING=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    KRIGING=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Regression Kriging
    ----------
    [statistics_kriging.3]\n
    Regression Kriging for grid interpolation from irregular sample points.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - PREDICTORS [`input grid list`] : Predictors
    - REGRESSION [`output grid`] : Regression. regression model applied to predictor grids
    - PREDICTION [`output grid`] : Prediction
    - RESIDUALS [`output grid`] : Residuals
    - VARIANCE [`output grid`] : Prediction Error
    - INFO_COEFF [`output table`] : Regression: Coefficients
    - INFO_MODEL [`output table`] : Regression: Model
    - INFO_STEPS [`output table`] : Regression: Steps
    - FIELD [`table field`] : Attribute
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] standard deviation [1] variance Default: 0
    - COORD_X [`boolean`] : Include X Coordinate. Default: 0
    - COORD_Y [`boolean`] : Include Y Coordinate. Default: 0
    - INTERCEPT [`boolean`] : Intercept. Default: 1
    - METHOD [`choice`] : Method. Available Choices: [0] include all [1] forward [2] backward [3] stepwise Default: 3
    - P_VALUE [`floating point number`] : Significance Level. Minimum: 0.000000 Maximum: 100.000000 Default: 5.000000 Significance level (aka p-value) as threshold for automated predictor selection, given as percentage
    - RESAMPLING [`choice`] : Resampling. Available Choices: [0] Nearest Neighbour [1] Bilinear Interpolation [2] Bicubic Spline Interpolation [3] B-Spline Interpolation Default: 3
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes for variogram estimation
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Variogram Model. Default: a + b * x
    - KRIGING [`choice`] : Kriging Type. Available Choices: [0] Simple Kriging [1] Ordinary Kriging Default: 0
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '3', 'Regression Kriging')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('PREDICTORS', PREDICTORS)
        Tool.Set_Output('REGRESSION', REGRESSION)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('RESIDUALS', RESIDUALS)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('INFO_COEFF', INFO_COEFF)
        Tool.Set_Output('INFO_MODEL', INFO_MODEL)
        Tool.Set_Output('INFO_STEPS', INFO_STEPS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('COORD_X', COORD_X)
        Tool.Set_Option('COORD_Y', COORD_Y)
        Tool.Set_Option('INTERCEPT', INTERCEPT)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('P_VALUE', P_VALUE)
        Tool.Set_Option('RESAMPLING', RESAMPLING)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('KRIGING', KRIGING)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False

def Variogram_Dialog(
    POINTS=None,
    VARIOGRAM=None,
    ATTRIBUTE=None,
    LOG=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None, Verbose=2):
    '''
    Variogram (Dialog)
    ----------
    [statistics_kriging.4]\n
    Variogram (Dialog)\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - VARIOGRAM [`output table`] : Variogram
    - ATTRIBUTE [`table field`] : Attribute
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Default: -1.000000
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: a + b * x

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '4', 'Variogram (Dialog)')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Output('VARIOGRAM', VARIOGRAM)
        Tool.Set_Option('ATTRIBUTE', ATTRIBUTE)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        return Tool.Execute(Verbose)
    return False

def run_tool_statistics_kriging_4(
    POINTS=None,
    VARIOGRAM=None,
    ATTRIBUTE=None,
    LOG=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None, Verbose=2):
    '''
    Variogram (Dialog)
    ----------
    [statistics_kriging.4]\n
    Variogram (Dialog)\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - VARIOGRAM [`output table`] : Variogram
    - ATTRIBUTE [`table field`] : Attribute
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Default: -1.000000
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: a + b * x

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '4', 'Variogram (Dialog)')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Output('VARIOGRAM', VARIOGRAM)
        Tool.Set_Option('ATTRIBUTE', ATTRIBUTE)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        return Tool.Execute(Verbose)
    return False

def Simple_Kriging_3D(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    Z_FIELD=None,
    Z_SCALE=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TARGET_USER_ZSIZE=None,
    TARGET_USER_ZMIN=None,
    TARGET_USER_ZMAX=None,
    TARGET_USER_ZNUM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Simple Kriging (3D)
    ----------
    [statistics_kriging.5]\n
    Simple Kriging interpolation for 3-dimensional data points. Output will be a grid collection with evenly spaced Z-levels representing the 3rd dimension.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTION [`output grid collection`] : Prediction
    - VARIANCE [`output grid collection`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - Z_FIELD [`table field`] : Z
    - Z_SCALE [`floating point number`] : Z Factor. Default: 1.000000
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TARGET_USER_ZSIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_ZMIN [`floating point number`] : Bottom. Default: 0.000000
    - TARGET_USER_ZMAX [`floating point number`] : Top. Default: 100.000000
    - TARGET_USER_ZNUM [`integer number`] : Levels. Minimum: 1 Default: 101
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '5', 'Simple Kriging (3D)')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('Z_FIELD', Z_FIELD)
        Tool.Set_Option('Z_SCALE', Z_SCALE)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TARGET_USER_ZSIZE', TARGET_USER_ZSIZE)
        Tool.Set_Option('TARGET_USER_ZMIN', TARGET_USER_ZMIN)
        Tool.Set_Option('TARGET_USER_ZMAX', TARGET_USER_ZMAX)
        Tool.Set_Option('TARGET_USER_ZNUM', TARGET_USER_ZNUM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False

def run_tool_statistics_kriging_5(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    Z_FIELD=None,
    Z_SCALE=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TARGET_USER_ZSIZE=None,
    TARGET_USER_ZMIN=None,
    TARGET_USER_ZMAX=None,
    TARGET_USER_ZNUM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Simple Kriging (3D)
    ----------
    [statistics_kriging.5]\n
    Simple Kriging interpolation for 3-dimensional data points. Output will be a grid collection with evenly spaced Z-levels representing the 3rd dimension.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTION [`output grid collection`] : Prediction
    - VARIANCE [`output grid collection`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - Z_FIELD [`table field`] : Z
    - Z_SCALE [`floating point number`] : Z Factor. Default: 1.000000
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TARGET_USER_ZSIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_ZMIN [`floating point number`] : Bottom. Default: 0.000000
    - TARGET_USER_ZMAX [`floating point number`] : Top. Default: 100.000000
    - TARGET_USER_ZNUM [`integer number`] : Levels. Minimum: 1 Default: 101
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '5', 'Simple Kriging (3D)')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('Z_FIELD', Z_FIELD)
        Tool.Set_Option('Z_SCALE', Z_SCALE)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TARGET_USER_ZSIZE', TARGET_USER_ZSIZE)
        Tool.Set_Option('TARGET_USER_ZMIN', TARGET_USER_ZMIN)
        Tool.Set_Option('TARGET_USER_ZMAX', TARGET_USER_ZMAX)
        Tool.Set_Option('TARGET_USER_ZNUM', TARGET_USER_ZNUM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False

def Ordinary_Kriging_3D(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    Z_FIELD=None,
    Z_SCALE=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TARGET_USER_ZSIZE=None,
    TARGET_USER_ZMIN=None,
    TARGET_USER_ZMAX=None,
    TARGET_USER_ZNUM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Ordinary Kriging (3D)
    ----------
    [statistics_kriging.6]\n
    Ordinary Kriging interpolation for 3-dimensional data points. Output will be a grid collection with evenly spaced Z-levels representing the 3rd dimension.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTION [`output grid collection`] : Prediction
    - VARIANCE [`output grid collection`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - Z_FIELD [`table field`] : Z
    - Z_SCALE [`floating point number`] : Z Factor. Default: 1.000000
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TARGET_USER_ZSIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_ZMIN [`floating point number`] : Bottom. Default: 0.000000
    - TARGET_USER_ZMAX [`floating point number`] : Top. Default: 100.000000
    - TARGET_USER_ZNUM [`integer number`] : Levels. Minimum: 1 Default: 101
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '6', 'Ordinary Kriging (3D)')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('Z_FIELD', Z_FIELD)
        Tool.Set_Option('Z_SCALE', Z_SCALE)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TARGET_USER_ZSIZE', TARGET_USER_ZSIZE)
        Tool.Set_Option('TARGET_USER_ZMIN', TARGET_USER_ZMIN)
        Tool.Set_Option('TARGET_USER_ZMAX', TARGET_USER_ZMAX)
        Tool.Set_Option('TARGET_USER_ZNUM', TARGET_USER_ZNUM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False

def run_tool_statistics_kriging_6(
    POINTS=None,
    TARGET_TEMPLATE=None,
    PREDICTION=None,
    VARIANCE=None,
    CV_SUMMARY=None,
    CV_RESIDUALS=None,
    Z_FIELD=None,
    Z_SCALE=None,
    FIELD=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    TARGET_USER_ZSIZE=None,
    TARGET_USER_ZMIN=None,
    TARGET_USER_ZMAX=None,
    TARGET_USER_ZNUM=None,
    TQUALITY=None,
    VAR_MAXDIST=None,
    VAR_NCLASSES=None,
    VAR_NSKIP=None,
    VAR_MODEL=None,
    LOG=None,
    BLOCK=None,
    DBLOCK=None,
    CV_METHOD=None,
    CV_SAMPLES=None,
    SEARCH_RANGE=None,
    SEARCH_RADIUS=None,
    SEARCH_POINTS_ALL=None,
    SEARCH_POINTS_MIN=None,
    SEARCH_POINTS_MAX=None, Verbose=2):
    '''
    Ordinary Kriging (3D)
    ----------
    [statistics_kriging.6]\n
    Ordinary Kriging interpolation for 3-dimensional data points. Output will be a grid collection with evenly spaced Z-levels representing the 3rd dimension.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Points
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - PREDICTION [`output grid collection`] : Prediction
    - VARIANCE [`output grid collection`] : Prediction Error
    - CV_SUMMARY [`output table`] : Cross Validation Summary
    - CV_RESIDUALS [`output shapes`] : Cross Validation Residuals
    - Z_FIELD [`table field`] : Z
    - Z_SCALE [`floating point number`] : Z Factor. Default: 1.000000
    - FIELD [`table field`] : Attribute
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - TARGET_USER_ZSIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_ZMIN [`floating point number`] : Bottom. Default: 0.000000
    - TARGET_USER_ZMAX [`floating point number`] : Top. Default: 100.000000
    - TARGET_USER_ZNUM [`integer number`] : Levels. Minimum: 1 Default: 101
    - TQUALITY [`choice`] : Error Measure. Available Choices: [0] Standard Deviation [1] Variance Default: 1
    - VAR_MAXDIST [`floating point number`] : Maximum Distance. Minimum: 0.000000 Default: 0.000000 maximum distance for variogram estimation, ignored if set to zero
    - VAR_NCLASSES [`integer number`] : Lag Distance Classes. Minimum: 1 Default: 100 initial number of lag distance classes
    - VAR_NSKIP [`integer number`] : Skip. Minimum: 1 Default: 1
    - VAR_MODEL [`text`] : Model. Default: b * x
    - LOG [`boolean`] : Logarithmic Transformation. Default: 0
    - BLOCK [`boolean`] : Block Kriging. Default: 0
    - DBLOCK [`floating point number`] : Block Size. Minimum: 0.000000 Default: 100.000000 Edge length [map units]
    - CV_METHOD [`choice`] : Cross Validation. Available Choices: [0] none [1] leave one out [2] 2-fold [3] k-fold Default: 0
    - CV_SAMPLES [`integer number`] : Cross Validation Subsamples. Minimum: 2 Default: 10 number of subsamples for k-fold cross validation
    - SEARCH_RANGE [`choice`] : Search Range. Available Choices: [0] local [1] global Default: 1
    - SEARCH_RADIUS [`floating point number`] : Maximum Search Distance. Minimum: 0.000000 Default: 1000.000000 local maximum search distance given in map units
    - SEARCH_POINTS_ALL [`choice`] : Number of Points. Available Choices: [0] maximum number of nearest points [1] all points within search distance Default: 1
    - SEARCH_POINTS_MIN [`integer number`] : Minimum. Minimum: 1 Default: 16 minimum number of points to use
    - SEARCH_POINTS_MAX [`integer number`] : Maximum. Minimum: 1 Default: 20 maximum number of nearest points

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('statistics_kriging', '6', 'Ordinary Kriging (3D)')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('CV_SUMMARY', CV_SUMMARY)
        Tool.Set_Output('CV_RESIDUALS', CV_RESIDUALS)
        Tool.Set_Option('Z_FIELD', Z_FIELD)
        Tool.Set_Option('Z_SCALE', Z_SCALE)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('TARGET_USER_ZSIZE', TARGET_USER_ZSIZE)
        Tool.Set_Option('TARGET_USER_ZMIN', TARGET_USER_ZMIN)
        Tool.Set_Option('TARGET_USER_ZMAX', TARGET_USER_ZMAX)
        Tool.Set_Option('TARGET_USER_ZNUM', TARGET_USER_ZNUM)
        Tool.Set_Option('TQUALITY', TQUALITY)
        Tool.Set_Option('VAR_MAXDIST', VAR_MAXDIST)
        Tool.Set_Option('VAR_NCLASSES', VAR_NCLASSES)
        Tool.Set_Option('VAR_NSKIP', VAR_NSKIP)
        Tool.Set_Option('VAR_MODEL', VAR_MODEL)
        Tool.Set_Option('LOG', LOG)
        Tool.Set_Option('BLOCK', BLOCK)
        Tool.Set_Option('DBLOCK', DBLOCK)
        Tool.Set_Option('CV_METHOD', CV_METHOD)
        Tool.Set_Option('CV_SAMPLES', CV_SAMPLES)
        Tool.Set_Option('SEARCH_RANGE', SEARCH_RANGE)
        Tool.Set_Option('SEARCH_RADIUS', SEARCH_RADIUS)
        Tool.Set_Option('SEARCH_POINTS_ALL', SEARCH_POINTS_ALL)
        Tool.Set_Option('SEARCH_POINTS_MIN', SEARCH_POINTS_MIN)
        Tool.Set_Option('SEARCH_POINTS_MAX', SEARCH_POINTS_MAX)
        return Tool.Execute(Verbose)
    return False




#________________________________________
def install(package):
    import subprocess, sys; subprocess.check_call([sys.executable, '-m', 'pip', 'install', package])

#________________________________________
def Create_Python_Toolboxes(dir_tool, dir_saga):
    print('\nPySAGA.tools creation started')

    import os, sys

    if not dir_saga:
        dir_saga = (os.path.dirname(__file__) + '/../../../bin/saga_x64')

    if not os.path.exists(dir_saga):
        print('Error: SAGA path does not exist: \'{:s}\''.format(dir_saga))
        return False

    dir_saga = os.path.abspath(dir_saga)
    print(' - SAGA directory is: \'{:s}\''.format(dir_saga))

    if not dir_tool:
        dir_tool = dir_saga + '/PySAGA/tools'

    dir_tool = os.path.abspath(dir_tool)
    print(' - Tool directory is: \'{:s}\''.format(dir_tool))

    sys.path.insert(1, dir_saga)
    from PySAGA import helper
    helper.Create_Toolboxes(dir_tool)

    print('\nPySAGA.tools creation finished')
    return True

#________________________________________
if __name__ == '__main__':
    import sys, os; dir_tool = None; dir_saga = None

    if len(sys.argv) > 2 and os.path.exists(sys.argv[2]):
        dir_saga = sys.argv[2]

    if len(sys.argv) > 1:
        dir_tool = sys.argv[1]

    Create_Python_Toolboxes(dir_tool, dir_saga)



#! /usr/bin/env python

#################################################################################
# MIT License

# Copyright (c) 2023 Olaf Conrad

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#################################################################################
# Purpose
#################################################################################

'''
The PySAGA.data.srtm module provides easy-to-use functions for accessing
``SRTM (Shuttle Radar Topography Mission)`` global elevation data.

Currently support is only given for the SRTM 90m DEM Digital Elevation
Database provided by CGIAR CSI. For more information on the data refer to
    https://srtm.csi.cgiar.org/

For downloading requested files the wget Python package is used, which needs to be
installed in order to work. Installation can be done through pip:
    pip install wget

Basic usage:
    import PySAGA.data.srtm as srtm, PySAGA.helper as saga_helper

    srtm.Dir_Global = 'C:/srtm_3arcsec/_global'

    srtm.CGIAR_Get_AOI(
        saga_helper.Get_AOI_From_Extent(279000, 920000, 5235000, 6102000, 32632),
        'C:/srtm_3arcsec/germany_utm32n_1000m.sg-grd-z', 1000
    )
'''


#################################################################################
#
# Globals
#________________________________________________________________________________

import os, PySAGA.saga_api as saga_api, PySAGA.helper, PySAGA.data.helper

Dir_Global = os.getcwd() + '/srtm_global'

VRT_Name = 'srtm_global'

Download_Retries = 4


#################################################################################
#
# Providing the original SRTM files...
#________________________________________________________________________________

#################################################################################
#
# CGIAR SRTM 3''...
#________________________________________________________________________________

#________________________________________________________________________________
# Returns 0 if tile is already present in the local storage path, 1 if it has
# been download successfully, or -1 if the download failed. Local storage path is
# defined by the global 'Dir_Global' variable and defaults to 'global' subfolder
# relative to the current working directory.
#________________________________________________________________________________
def CGIAR_Get_Tile(Col, Row, DeleteZip=True):
    '''
    Downloads the ``SRTM 3arcsec`` tile for the specified column (range 1 to 72)
    and row (range 1 to 24).
    Returns 0 if tile is already present in the local storage path, 1 if it has
    been download successfully, or -1 if the download failed. Local storage path is
    defined by the global 'Dir_Global' variable and defaults to 'global' subfolder
    relative to the current working directory.
    '''

    if Col < 1 or Col > 72:
        saga_api.SG_UI_Console_Print_StdErr('requested column {:d} is out-of-range (1-72)'.format(Row))
        return -1

    if Row < 1 or Row > 24:
        saga_api.SG_UI_Console_Print_StdErr('requested row {:d} is out-of-range (1-24)'.format(Col))
        return -1

    Local_File = '{:s}/srtm_{:02d}_{:02d}.tif'.format(Dir_Global, Col, Row)
    if os.path.exists(Local_File):
        return 0

    Remote_Dir = 'https://srtm.csi.cgiar.org/wp-content/uploads/files/srtm_5x5/TIFF'

    Zip_File = PySAGA.data.helper.Get_File('srtm_{:02d}_{:02d}.zip'.format(Col, Row), Dir_Global, Remote_Dir)
    if not Zip_File:
        return -1

    import zipfile; zf = zipfile.ZipFile(Zip_File, 'r')
    Local_File = zf.extract('srtm_{:02d}_{:02d}.tif'.format(Col, Row), Dir_Global)
    zf.close()
    if DeleteZip:
        os.remove(Zip_File)
    return 1

#________________________________________________________________________________
def CGIAR_Get_Tiles(Cols=[1, 1], Rows=[1, 1], DeleteZip=True):
    '''
    Downloads all SRTM 3arcsec tiles for the given column and row span.
    '''

    if Cols[1] < Cols[0]:
        Col = Cols[0]; Cols[0] = Cols[1]; Cols[1] = Col
    if Cols[0] < 1:
        Cols[0] = 1
    elif Cols[1] > 72:
        Cols[1] = 72

    if Rows[1] < Rows[0]:
        Row = Rows[0]; Rows[0] = Rows[1]; Cols[1] = Row
    if Rows[0] < 1:
        Rows[0] = 1
    elif Rows[1] > 24:
        Rows[1] = 24

    saga_api.SG_UI_Msg_Add('requesting tiles for rows {:d}-{:d} and columns {:d}-{:d}\n'.format(Rows[0], Rows[1], Cols[0], Cols[1]))
    nAdded = 0; nFailed = 0; nFound = 0
    for Col in range(Cols[0], Cols[1] + 1):
        for Row in range(Rows[0], Rows[1] + 1):
            if Col >= 1 and Col <= 72 and Row >= 1 and Row <= 24:
                Result = CGIAR_Get_Tile(Col, Row, DeleteZip)
                if Result > 0:
                    nAdded  += 1
                elif Result < 0:
                    nFailed += 1
                else:
                    nFound  += 1

    if nFailed > 0:
        saga_api.SG_UI_Console_Print_StdErr('{:d} download(s) of {:d} failed'.format(nFailed, nFailed + nAdded))

    if nAdded > 0 or not os.path.exists('{:s}/{:s}.vrt'.format(Dir_Global, VRT_Name)):
        PySAGA.helper.Get_VRT(Dir_Global, 'tif', VRT_Name)

    return nAdded + nFound > 0

#________________________________________________________________________________
def CGIAR_Get_Tiles_byExtent(Lon=[-180, 180], Lat=[-60, 60], DeleteZip=True):
    '''
    Downloads all SRTM 3arcsec tiles for the given longitudinal and latitudinal range.
    '''

    Cellsize = 3. / 3600.; Lon[0] -= Cellsize; Lon[1] += Cellsize; Lat[0] -= Cellsize; Lat[1] += Cellsize

    Cols = [1 + int((Lon[0] + 180.) / 5.), 1 + int((Lon[1] + 180.) / 5.)]
    Rows = [1 + int(( 60. - Lat[1]) / 5.), 1 + int(( 60. - Lat[0]) / 5.)]

    return CGIAR_Get_Tiles(Cols, Rows, DeleteZip)

#________________________________________________________________________________
def CGIAR_Get_AOI(AOI, Target_File, Target_Resolution=90., Round=True, DeleteZip=True, Verbose=False):
    '''
    Downloads, clips and projects ``SRTM 3arcsec`` data matching the desired area of interest
    (*AOI*) including the resampling to the specified *Target_Resolution*.
    '''

    #____________________________________________________________________________
    def Round_Extent(Extent, Resolution):
        if Round:
            from math import floor, ceil
            Extent.xMin = Resolution * floor(Extent.xMin / Resolution)
            Extent.xMax = Resolution * ceil (Extent.xMax / Resolution)
            Extent.yMin = Resolution * floor(Extent.yMin / Resolution)
            Extent.yMax = Resolution * ceil (Extent.yMax / Resolution)
        return True

    def Import_Raster():
        if not AOI or not AOI.is_Valid() or not AOI.Get_Projection().is_Okay():
            saga_api.SG_UI_Console_Print_StdErr('invalid AOI')
            return None

        Extent = saga_api.CSG_Rect(AOI.Get_Extent())
        if not AOI.Get_Projection().is_Geographic():
            Round_Extent(Extent, Target_Resolution)
            _AOI = saga_api.CSG_Shapes(saga_api.SHAPE_TYPE_Point); _AOI.Get_Projection().Create(AOI.Get_Projection())
            _AOI.Add_Shape().Add_Point(Extent.Get_XMin   (), Extent.Get_YMin   ())
            _AOI.Add_Shape().Add_Point(Extent.Get_XMin   (), Extent.Get_YCenter())
            _AOI.Add_Shape().Add_Point(Extent.Get_XMin   (), Extent.Get_YMax   ())
            _AOI.Add_Shape().Add_Point(Extent.Get_XCenter(), Extent.Get_YMax   ())
            _AOI.Add_Shape().Add_Point(Extent.Get_XMax   (), Extent.Get_YMax   ())
            _AOI.Add_Shape().Add_Point(Extent.Get_XMax   (), Extent.Get_YCenter())
            _AOI.Add_Shape().Add_Point(Extent.Get_XMax   (), Extent.Get_YMin   ())
            _AOI.Add_Shape().Add_Point(Extent.Get_XCenter(), Extent.Get_YMin   ())
            if not saga_api.SG_Get_Projected(_AOI, None, saga_api.CSG_Projection.Get_GCS_WGS84()):
                saga_api.SG_UI_Console_Print_StdErr('failed to project AOI to GCS')
                return None
            Extent = _AOI.Get_Extent()#; Extent.Inflate(10 * 3 / 3600, False)

        Lon = [Extent.Get_XMin(), Extent.Get_XMax()]; Lat=[Extent.Get_YMin(), Extent.Get_YMax()]

        #________________________________________________________________________
        if not CGIAR_Get_Tiles_byExtent(Lon, Lat, DeleteZip):
            saga_api.SG_UI_Console_Print_StdErr('failed to update virtual raster tiles for requested extent')
            return None

        #________________________________________________________________________
        Tool = saga_api.SG_Get_Tool_Library_Manager().Get_Tool('io_gdal', '0')
        if not Tool:
            saga_api.SG_UI_Console_Print_StdErr('failed to request tool \'{:s}\''.format('Import Raster'))
            return None

        Tool.Reset()
        Tool.Set_Parameter('FILES', '{:s}/{:s}.vrt'.format(Dir_Global, VRT_Name))
        Tool.Set_Parameter('EXTENT', 1) # 'user defined'
        Tool.Set_Parameter('EXTENT_XMIN', Lon[0])
        Tool.Set_Parameter('EXTENT_XMAX', Lon[1])
        Tool.Set_Parameter('EXTENT_YMIN', Lat[0])
        Tool.Set_Parameter('EXTENT_YMAX', Lat[1])

        if not Tool.Execute():
            saga_api.SG_UI_Console_Print_StdErr('failed to execute tool \'{:s}\''.format(Tool.Get_Name().c_str()))
            return None

        Grid = Tool.Get_Parameter('GRIDS').asGridList().Get_Grid(0)
        if AOI.Get_Projection().is_Geographic():
            return Grid

        #________________________________________________________________________
        Tool = saga_api.SG_Get_Tool_Library_Manager().Get_Tool('pj_proj4', '4')
        if not Tool:
            saga_api.SG_UI_Console_Print_StdErr('failed to request tool \'{:s}\''.format('Coordinate Transformation (Grid)'))
            saga_api.SG_Get_Data_Manager().Delete(Grid)
            return None

        Extent = saga_api.CSG_Rect(AOI.Get_Extent()); Round_Extent(Extent, Target_Resolution)

        Tool.Reset()
        Tool.Set_Parameter('CRS_METHOD', 0)  # 'Proj4 Parameters'
        Tool.Set_Parameter('CRS_STRING', AOI.Get_Projection().Get_WKT())
        Tool.Set_Parameter('SOURCE'    , Grid)
        Tool.Set_Parameter('RESAMPLING', 3)  # B-Spline
        Tool.Set_Parameter('DATA_TYPE' , 10) # Preserve
        Tool.Set_Parameter('TARGET_DEFINITION', 0) # 'user defined'
        Tool.Set_Parameter('TARGET_USER_SIZE', Target_Resolution)
        Tool.Set_Parameter('TARGET_USER_XMAX', Extent.xMax)
        Tool.Set_Parameter('TARGET_USER_XMIN', Extent.xMin)
        Tool.Set_Parameter('TARGET_USER_YMAX', Extent.yMax)
        Tool.Set_Parameter('TARGET_USER_YMIN', Extent.yMin)

        if not Tool.Execute():
            saga_api.SG_UI_Console_Print_StdErr('failed to execute tool \'{:s}\''.format(Tool.Get_Name().c_str()))
            saga_api.SG_Get_Data_Manager().Delete(Grid)
            return None

        saga_api.SG_Get_Data_Manager().Delete(Grid)

        return Tool.Get_Parameter('GRID').asGrid()


    #############################################################################
    #____________________________________________________________________________
    saga_api.SG_UI_Console_Print_StdOut('processing: {:s}... '.format(Target_File))

    if not Verbose:
        saga_api.SG_UI_ProgressAndMsg_Lock(True) # suppress noise

    Grid = Import_Raster(); Result = False
    if Grid:
        Target_Dir = os.path.split(Target_File)[0]
        if not os.path.exists(Target_Dir):
            os.makedirs(Target_Dir)

        Result = Grid.Save(Target_File)

        saga_api.SG_Get_Data_Manager().Delete(Grid) # free memory

    if not Verbose:
        saga_api.SG_UI_ProgressAndMsg_Lock(False)
        saga_api.SG_UI_Console_Print_StdOut('okay' if Result else 'failed', False)

    return Result


#################################################################################
#
#________________________________________________________________________________



# Note on my machine the gnu pretty-printers are in this file /usr/share/gcc-xx.x.x/python/libstdcxx/printers.py

import gdb
import datetime

class array_printer:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        address = int(self.val['m_Values'])
        count = int(self.val['m_nValues'])
        buffer = int(self.val['m_nBuffer'])
        element_size = int(self.val['m_Value_Size'])

        data = gdb.selected_inferior().read_memory(address, count * element_size)
        
        # Hier müssen Sie die Daten basierend auf der Größe jedes Elements interpretieren
        data_str = ''
        for i in range(0, len(data), element_size):
            data_str += data[i:i+element_size].hex()  # ))#, 'little')) 
            if(i+element_size != len(data)):
                data_str += ', '

        return 'CSG_Array: {size = ' + str(count) + ', buffer = ' + str(buffer) + ', values = {' + data_str + '}}'


    def dispay_hint(self):
        return 'array'

class array_int_printer:
    def __init__(self, val):
        self.val = val
        self.address = int(self.val['m_Array']['m_Values'])
        self.count = int(self.val['m_Array']['m_nValues'])
        self.buffer = int(self.val['m_Array']['m_nBuffer'])
        self.element_size = int(self.val['m_Array']['m_Value_Size'])

    def values(self):
        int_t = gdb.lookup_type('int').pointer()
        values = ''
        for i in range(self.count):
            element = gdb.Value(self.address + i*self.element_size ).cast(int_t).dereference()
            values += str(element)
            if( i+1 != self.count ):
                values += ', '

        return values

    def to_string(self):
        return 'CSG_Array_Int: {size = ' + str(self.count) + ', buffer = ' + str(self.buffer) + ', values = {' + self.values() + '}}'

    def dispay_hint(self):
        return 'array'

class array_pointer_printer:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        address = int(self.val['m_Array']['m_Values'])
        count = int(self.val['m_Array']['m_nValues'])
        buffer = int(self.val['m_Array']['m_nBuffer'])
        element_size = int(self.val['m_Array']['m_Value_Size'])

        data = gdb.selected_inferior().read_memory(address, count * element_size)
        
        # Hier müssen Sie die Daten basierend auf der Größe jedes Elements interpretieren
        data_str = ''
        for i in range(0, len(data), element_size):
            data_str += str(hex(int.from_bytes(data[i:i+element_size], 'little')))  # ))#, 'little')) 
            #data_str += str(read_data(address, count, element_size)  # ))#, 'little')) 
            if(i+element_size != len(data)):
                data_str += ', '

        return 'CSG_Array_Pointer: {size = ' + str(count) + ', buffer = ' + str(buffer) + ', values = {' + data_str + '}}'


    def dispay_hint(self):
        return 'array'

class data_object_printer:
    def __init__(self, val):
        self.val = val

    def format(self):
        name = str(self.val['m_Name'])
        file_name = str(self.val['m_FileName'])
        nodata_value_lo = str(self.val['m_NoData_Value'][0])
        nodata_value_hi = str(self.val['m_NoData_Value'][1])
        modified = str(self.val['m_bModified'])
        ref_id = str(self.val['m_RefID'])
        managed = str(self.val['m_Managed'])
        max_samples = str(self.val['m_Max_Samples'])

        projection = projection_printer(self.val['m_Projection']).format()

        return 'name = ' + name + ', file name: ' + file_name + ', nodata value ={' + nodata_value_lo + ', ' + nodata_value_hi + '}, max saples: ' + max_samples + ', projection ={'+ projection + '} ref id = ' + ref_id + ', managed = ' + managed + ', medified = ' + modified  

    def to_string(self):
        return 'CSG_Data_Object: {' + self.format() + '}'

class datetime_printer:
    def __init__(self, val):
        self.val = val
        self.time = self.val['m_pDateTime']['m_time']['m_ll']

    def value(self):
        time = str(self.time)[:10]
        try:
            dt = datetime.datetime.fromtimestamp(int(time),datetime.timezone.utc)
            time = '{:%Y-%m-%d %H:%M:%S}'.format(dt)
        except:
            pass
        return time

    def to_string(self):
        return 'CSG_DateTime: iso 8601 = ' + self.value() + ', unix = ' + str(self.time)

class grid_printer:
    def __init__(self, val):
        self.val = val
        self.offset = self.val['m_zOffset']
        self.scale = self.val['m_zScale']
        self.data_type = self.val['m_Type'] # = SG_DATATYPE_Float, 
        self.unit = self.val['m_Unit'] 
        self.stat = self.val['m_Statistics']
        self.system = self.val['m_System']

    def format(self):
        return "data type = %s, unit = %s, scale = %f, offset = %f, statistic ={%s} "  % ( str(self.data_type)[12:], str(self.unit), self.scale, self.offset, simple_statistics_printer(self.stat).format() )

    def to_string(self):
        return "CSG_Grid: {%s, grid system ={%s}, data object ={%s}}" % ( self.format(), grid_system_printer(self.system).format(), data_object_printer(self.val).format() )

class grid_system_printer:
    def __init__(self, val):
        self.val = val

    def format(self):
        columns = int(self.val['m_NX'])
        rows = int(self.val['m_NY'])
        count = int(self.val['m_NCells'])
        size = float(self.val['m_Cellsize'])
        area = float(self.val['m_Cellarea'])
        extent = self.val['m_Extent']
        return "columns = %d, rows = %d, count = %d, cellsize = %f, cellarea = %f, extent = {%s}" % ( columns, rows, count, size, area, rect_printer(extent).format() )

    def to_string(self):
        return 'CSG_Grid_System: {' + self.format() + '}'


class parameter_printer:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        ident = str(self.val['m_Identifier'])
        name = str(self.val['m_Name'])
        type = self.val.dynamic_type 
        type_string = str(type)[14:]
        description = str(self.val['m_Description'])
        enabled = str(self.val['m_bEnabled'])
        parameters = str(self.val['m_pParameters'])
        parent = str(self.val['m_pParent'])
        nchild = str(self.val['m_nChildren'])
        children = str(self.val['m_Children'])


            # value_string += str( polymorph['m_pMin'].dereference() )
        
        return 'CSG_Parameter = Identifier = ' + ident + ', Name = ' + name + ', Description = ' + description + ', Enabled = ' + enabled +', Type = {' + self.value() + '}, Parameters = ' + parameters + ', Parent = ' + parent + ', Children Count = ' + nchild + ', Children = ' + children

    def value(self):
        default = str(self.val['m_Default'])
        type = self.val.dynamic_type 
        type_string = str(type)[14:]
        polymorph = self.val.cast(type)

        value_string = ''
        if( type_string == "Node" ):
            value_string = 'Node'

        if( type_string == "Bool" ):
            value_string = 'Bool = ' + str(polymorph['m_Value'])


        if( type_string == "Double" or type_string == "Int"):
            value_string = type_string + ' = ' + str(polymorph['m_Value'])
            if( bool(polymorph['m_bMinimum']) == True ):
                value_string += ', Minimum = ' + str(polymorph['m_Minimum'])
            else:
                value_string += ', Minimum = ' + str(polymorph['m_bMinimum'])
            if( bool(polymorph['m_bMaximum']) == True ):
                value_string += ', Maximum = ' + str(polymorph['m_Maximum'])
            else:
                value_string += ', Maximum = ' + str(polymorph['m_bMaximum'])

            #value_string += str(default)
        
        if( type_string == "Range" ):

            range_type = polymorph['m_pMin'].dynamic_type 
            min_val = polymorph['m_pMin'].cast(range_type)
            max_val = polymorph['m_pMax'].cast(range_type)

            value_string = 'Range = {' + str(min_val['m_Value']) + ', ' + str(max_val['m_Value']) + '}'

            if( bool(min_val['m_bMinimum']) == True ):
                value_string += ', Minimum = ' + str(min_val['m_Minimum'])
            else:
                value_string += ', Minimum = ' + str(min_val['m_bMinimum'])
            if( bool(min_val['m_bMaximum']) == True ):
                value_string += ', Maximum = ' + str(min_val['m_Maximum'])
            else:
                value_string += ', Maximum = ' + str(min_val['m_bMaximum'])

        if( type_string == "Choice" ):
            value_string = 'Choice = ' + str(polymorph['m_Value']) + ', Items = {' + strings_printer(polymorph['m_Items']).values() + '}'

        if( type_string == "Choices" ):
            selection = str(array_int_printer(polymorph['m_Selection']).values())
            items = str(strings_printer(polymorph['m_Items'][0]).values())
            data = str(strings_printer(polymorph['m_Items'][1]).values())
            value_string = 'Choices ={' + selection + '}, Items ={' + items + '}, Data ={' + data + '}'

        if( type_string == "String" ):
            value_string = 'String = ' + string_printer(polymorph['m_String']).to_string() + ', Default = ' + default 

        if( type_string == "Date" ):
            value_string = 'Date = ' + datetime_printer(polymorph['m_Date']).value() + ', Default = ' + default 

        return value_string

class parameters_printer:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        return 'Identifier = ' + str(self.val['m_Identifier'])

class projection_printer:
    def __init__(self, val):
        self.val = val

    def format(self):
        name = str(self.val['m_Name'])
        authority_name = str(self.val['m_Authority'])
        authority_id = str(self.val['m_Authority_ID'])
        projection_type = str(self.val['m_Type'])
        projection_type = str(projection_type[16:])
        projection_unit = str(self.val['m_Unit'])
        projection_unit = str(projection_unit[13:])
        proj_string = str(self.val['m_Proj4'])
        wkt_string = str(self.val['m_WKT'])

        return 'name = ' + name + ', authority = {' + authority_name + ',  ' + authority_id + '}, type = ' + projection_type + ', unit = ' + projection_unit + ', proj = ' + proj_string + ', wkt = ' + wkt_string

    def to_string(self):
        return 'CSG_Projection: {' + self.format() + '}'

class rect_printer:
    def __init__(self, val):
        self.val = val

    def format(self):
        xmin = self.val['xMin']
        ymin = self.val['yMin']
        xmax = self.val['xMax']
        ymax = self.val['yMax']
        return "left = %f, right = %f, up = %f, down = %f" % ( xmin, xmax, ymax, ymin )

    def to_string(self):
        return "CSG_Rect: {%s}" % ( self.format() )

class shape_printer:
    def __init__(self, val):
        self.val = val

        #self.val = val.cast(gdb.lookup_type('CSG_Shape_Points'))
    def to_string(self):
        obj_type = self.val.dynamic_type 
        cast = self.val.cast(obj_type)

        if( obj_type == gdb.lookup_type('CSG_Shape_Point') ):
            return shape_point_printer(cast).to_string()
            #return "Printer"

        if( obj_type == gdb.lookup_type('CSG_Shape_Point').pointer() ):
            return shape_point_printer(cast).to_string()

        if( obj_type == gdb.lookup_type('CSG_Shape_Points') ):
            return shape_points_printer(cast).to_string()
            #return "Printer"

        if( obj_type == gdb.lookup_type('CSG_Shape_Points').pointer() ):
            return shape_points_printer(cast).to_string()
            #return "Printer"
        return "shape_printer successfully " + str(obj_type)

class shape_part_printer:
    def __init__(self, val):
        self.val = val
        self.count = int(self.val['m_nPoints'])
        self.points = self.val['m_Points']
        self.zvals = self.val['m_Z']
        self.mvals = self.val['m_M']

    def to_string(self):
        # shape_part_p_t = gdb.lookup_type('CSG_Shape_Part').pointer()
        #string_p_t = gdb.lookup_type('CSG_String').pointer()
        # string_ptr = gdb.Value(self.parts + i*shape_part_p_t.sizeof ).reinterpret_cast(void_pp_t).dereference().reinterpret_cast(string_p_t)
        tsg_point_p_t = gdb.lookup_type('TSG_Point').pointer()

        values = ''
        #string_p_t = gdb.lookup_type('CSG_String').pointer()
        # string_ptr = gdb.Value(self.parts + i*shape_part_p_t.sizeof ).reinterpret_cast(void_pp_t).dereference().reinterpret_cast(string_p_t)

        values += self.val.type.name
        #values += str(self.parts)
        for i in range(self.count):

            values += 'test'
            #point = gdb.Value(self.points + i*tsg_point_p_t.sizeof ).dereference()
            #values += str(point['x']) + ', ' + str(point['y'])
        #values += str(self.parts)

        return str(self.points.type) + str(self.count)

        for i in range(self.count):
            part_ptr = gdb.Value(self.parts + i*shape_part_p_t.sizeof ).cast(shape_part_p_t)
            values += str( part_ptr )
            values += str( part_ptr.type )
        return "shape_part_printer successfully "

"""
{_vptr.CSG_Shape_Part = 0x7ffff6e0c968 <vtable for CSG_Shape_Part+16>, 
    m_bUpdate = false, 
    m_nPoints = 28, 
    m_nBuffer = 28, 
    m_Z = 0x55555eef3520, 
    m_ZMin = 0, 
    m_ZMax = 14.666666666666666, 
    m_M = 0x55555ef5bfd0, 
    m_MMin = 0,
    m_MMax = 14.66666666666667, 
    m_Points = 0x55555f3318c0, 
    m_Extent = {<SSG_Rect> = {xMin = 0.35616099197477324, yMin = 0.61964419765712786, xMax = 0.72878122881547802, yMax = 0.92463353636269607}, <No data fields>},
    m_pOwner = 0x55555e653720}
"""

class shape_point_printer:
    def __init__(self, val):
        self.val = val
        self.x = self.val['m_Point']['x']
        self.y = self.val['m_Point']['y']

    def to_string(self):
        return "CSG_Shape_Point: {point ={x = %f, y = %f}, table record ={%s}" % ( self.x, self.y, table_record_printer(self.val).format() )

class shape_point_z_printer:
    def __init__(self, val):
        self.val = val
        self.x = self.val['m_Point']['x']
        self.y = self.val['m_Point']['y']
        self.z = self.val['m_Z']

    def to_string(self):
        return "CSG_Shape_Point_Z: {point ={x = %f, y = %f, z = %f}, table record ={%s}" % ( self.x, self.y, self.z, table_record_printer(self.val).format() )

class shape_point_zm_printer:
    def __init__(self, val):
        self.val = val
        self.x = self.val['m_Point']['x']
        self.y = self.val['m_Point']['y']
        self.z = self.val['m_Z']
        self.m = self.val['m_M']

    def to_string(self):
        return "CSG_Shape_Point_ZM: {point ={x = %f, y = %f, z = %f, m = %f}, table record ={%s}" % ( self.x, self.y, self.z, self.m, table_record_printer(self.val).format() )

class shape_points_printer:
    def __init__(self, val):
        self.val = val
        self.part_count = int(self.val['m_nParts'])
        self.parts = self.val['m_pParts']
        self.points = int(self.val['m_nPoints'])
        self.zmin = float(self.val['m_ZMin'])
        self.zmax = float(self.val['m_ZMax'])
        self.mmin = float(self.val['m_MMin'])
        self.mmax = float(self.val['m_MMax'])
        self.extent = self.val['m_Extent']
        self.table = self.val['m_pTable']
        #self.update = self.val['m_bUpdate']

    def to_string(self):

        shape_part_p_t = gdb.lookup_type('CSG_Shape_Part').pointer()
        #string_p_t = gdb.lookup_type('CSG_String').pointer()
        # string_ptr = gdb.Value(self.parts + i*shape_part_p_t.sizeof ).reinterpret_cast(void_pp_t).dereference().reinterpret_cast(string_p_t)

        values = ''
        #values += str(self.parts)
        for i in range(self.part_count):
            part_ptr = gdb.Value(self.parts + i*shape_part_p_t.sizeof ).cast(shape_part_p_t)
            values = shape_part_printer(part_ptr).to_string()
            #values += str( part_ptr )
            #values += str( part_ptr.type )
            #values += str(string_printer(string_ptr.dereference()).to_string())
            #if( i+1 < self.count ):
             #   values += ', '

        return values

        obj_type = gdb.lookup_type('CSG_Shapes').pointer()
        vertex_type = str(self.table.cast(obj_type)['m_Vertex_Type'])[15:]

        optional_zm = ''
        if( vertex_type == 'XYZ' or vertex_type == 'XYZM' ):
            optional_zm += ", z min = %f, z max = %f" % ( self.zmin, self.zmax )
        if( vertex_type == 'XYZM' ):
            optional_zm += ", m min = %f, m max = %f" % ( self.mmin, self.mmax )

        return "CSG_Shape_Points: {parts = %d, points = %d, exten ={%s}%s, table record ={%s}, parts ={%s}}" % ( self.part_count, self.points, rect_printer(self.extent).format(), optional_zm, table_record_printer(self.val).format() )

    """
CSG_Shape_Points: {
    parts = 3, points = 3, 
    exten ={
        left = 0.356161, right = 0.728781, up = 0.924634, down = 0.619644
    }, 
    z min = 0.000000, z max = 0.329412, m min = 0.000000, m max = 0.000000, table 
    record ={index = 0, 
        fields = 2 {
            1, "One"
        }
    }
}}
{<CSG_Shape> = {
    <CSG_Table_Record> = {_vptr.CSG_Table_Record = 0x7ffff6e0cb88 <vtable for CSG_Shape_Points+16>, 
        m_Flags = 0 '\000', 
        m_Index = 0, 
        m_Values = 0x55555f4ab810, 
        m_pTable = 0x55555e8212b0
    },
    <No data fields>
},
m_bUpdate = false, 
m_nParts = 3, 
m_nPoints = 28, 
m_ZMin = 8.6916947597937554e-311, 
m_ZMax = 0.32941176470588235, 
m_MMin = 0, 
m_MMax = 1.6296927674613291e-310, 
m_Extent = {<SSG_Rect> = {xMin = 0.35616099197477324,
      yMin = 0.61964419765712786, xMax = 0.72878122881547802, yMax = 0.92463353636269607}, <No data fields>}, m_pParts = 0x55555f158700}
    """

class shapes_printer:
    def __init__(self, val):
        self.val = val

    def format(self):
        shapes_type = self.val['m_Type']
        shapes_type_string = str(shapes_type)[11:]
        vertex_type = self.val['m_Vertex_Type']
        vertex_type_string = str(vertex_type)[15:]
        zmin = int(self.val['m_ZMin'])
        zmax = int(self.val['m_ZMax'])
        mmin = int(self.val['m_ZMin'])
        mmax = int(self.val['m_ZMax'])
        s = "shape type = %s, vertex type = %s" % (shapes_type_string, vertex_type_string )
        if( vertex_type_string == 'XYZ' or vertex_type_string == 'XYZM' ):
            s += ", z min = %f, z max = %f" % ( zmin, zmax ) 
        if( vertex_type_string == 'XYZM' ):
            s += ", m min = %f, m max = %f" % ( mmin, mmax ) 
        return s

    def to_string(self):
        return "CSG_Shapes: {%s, table ={%s}, data object ={%s}}" % ( self.format(), table_printer(self.val).format(), data_object_printer(self.val).format() )

class simple_statistics_printer:
    def __init__(self, val):
        self.val = val
        self.count = self.val['m_nValues']
        self.sorted = bool(self.val['m_bSorted'])
        self.eval_level = int(self.val['m_bEvaluated'])
        self.size = int(self.val['m_nValues'])
        self.weights = float(self.val['m_Weights'])
        self.sum = float(self.val['m_Sum'])
        #self.sum2 = float(self.val['m_Sum2'])
        self.min = float(self.val['m_Minimum'])
        self.max = float(self.val['m_Maximum'])
        self.range = float(self.val['m_Range'])
        self.mean = float(self.val['m_Mean'])
        self.variance = float(self.val['m_Variance'])
        self.std_dev = float(self.val['m_StdDev'])
        self.kurtosis = float(self.val['m_Kurtosis'])
        self.skewness = float(self.val['m_Skewness'])
        self.gini = float(self.val['m_Gini'])

        self.hold = bool(self.val['m_Values']['m_nValues'])

    def format(self):
        return "hold values = %s, sorted = %s, evaluation level = %d, size = %d, weights = %f, sum = %f, minimum = %f, maximum = %f, range = %f, mean = %f, variance = %f, standard deviation = %f, kurtosis = %f, skewness = %f, gini = %f" % (str(self.hold), str(self.sorted), self.eval_level, self.size, self.weights, self.sum, self.min, self.max, self.range, self.mean, self.variance, self.std_dev, self.kurtosis, self.skewness, self.gini)

    def to_string(self):
        return "CSG_Simple_Statistics: {%s}" % (self.format() )

class string_printer:
    def __init__(self, val):
        self.val = val
        self.string = str(self.val['m_pString']['m_impl'])

    def to_string(self):
        return str(self.string[1:]) 

class strings_printer:
    def __init__(self, val):
        self.val = val
        self.address = self.val['m_Strings']['m_Array']['m_Values']
        self.size = int(self.val['m_Strings']['m_Array']['m_Value_Size'])
        self.count = int(self.val['m_Strings']['m_Array']['m_nValues'])

    def values(self):
        void_pp_t = gdb.lookup_type('void').pointer().pointer()
        string_p_t = gdb.lookup_type('CSG_String').pointer()

        values = ''
        for i in range(self.count):
            string_ptr = gdb.Value(self.address + i*self.size ).reinterpret_cast(void_pp_t).dereference().reinterpret_cast(string_p_t)
            values += str(string_printer(string_ptr.dereference()).to_string())
            if( i+1 < self.count ):
                values += ', '

        return values

    def to_string(self):

        return "CSG_Strings: {size = %d, strings = {%s}}" % ( self.count, self.values() )

class table_printer:
    def __init__(self, val):
        self.val = val

    def format(self):

        n_fields = int(self.val['m_nFields'])
        n_records = int(self.val['m_nRecords'])
       
        fields = ''
        for i in range(n_fields):
            field_type = self.val['m_Field_Type'][i]
            field_type_string = str(field_type)[12:]
            field_name = self.val['m_Field_Name'][i]
            field_name_string = string_printer(field_name).to_string()
            fields += field_name_string + ' ' + field_type_string 
            if( i+1 != n_fields ):
                fields += ', '

        return "records = %d, fields = %d {%s}" % ( n_records, n_fields, fields )

    def to_string(self):
        return "CSG_Table: {%s, data object = {%s}}" % ( self.format(), data_object_printer(self.val).format() )


class table_record_printer:
    def __init__(self, val):
        self.val = val
        self.index = int(self.val['m_Index'])
        self.fields = int(self.val['m_pTable']['m_nFields'])

    def format(self):
        values = ""
        for i in range(self.fields):
            value = self.val['m_Values'][i]
            obj_type = gdb.lookup_type(str(value.dereference().dynamic_type))
            values += str(value.dereference().cast(obj_type)['m_Value'])
            if( i+1 != self.fields ):
                values += ', '

        return "index = %d, fields = %d {%s}}" % ( self.index, self.fields, values )


    def to_string(self):
        obj_type = self.val.type 

        if( obj_type == gdb.lookup_type('CSG_Shape_Point') ):
            return shape_point_printer(self.val).to_string()

        if( obj_type == gdb.lookup_type('CSG_Shape_Point').pointer() ):
            return shape_point_printer(self.val).to_string()

        if( obj_type == gdb.lookup_type('CSG_Shape_Point_Z') ):
            return shape_point_z_printer(self).to_string()

        if( obj_type == gdb.lookup_type('CSG_Shape_Point_Z').pointer() ):
            return shape_point_z_printer(self).to_string()

        if( obj_type == gdb.lookup_type('CSG_Shape_Point_ZM') ):
            return shape_point_zm_printer(self).to_string()

        if( obj_type == gdb.lookup_type('CSG_Shape_Point_ZM').pointer() ):
            return shape_point_zm_printer(self).to_string()

        if( obj_type == gdb.lookup_type('CSG_Shape_Points') ):
            return shape_point_printer(self.val).to_string()
            #return "Printer"

        if( obj_type == gdb.lookup_type('CSG_Shape_Points').pointer() ):
            return shape_point_printer(self.val).to_string()
            #return "Printer"

        return "CSG_Table_Record: {%s}" % self.format()


class meta_data_printer:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        name = str(self.val['m_Name'])
        content = str(self.val['m_Content'])
        parent = str(self.val['m_pParent'])
        dummy = str(self.val['m_pDummy'])
        children = int(self.val['m_Children']['m_nValues'])
        
        properties_names = str(strings_printer(self.val['m_Prop_Names']).values())
        properties_values = str(strings_printer(self.val['m_Prop_Values']).values())

        return "CSG_MetaData: {name = %s, content = %s, properties ={names ={%s}, values ={%s}}, parent = %s, dummy = %s, children = %d}" % ( name, content, properties_names, properties_values, parent, dummy, children )

def my_pp_func(val):
    if str(val.type) == 'CSG_Array': return array_printer(val)
    if str(val.type) == 'CSG_Array_Int': return array_int_printer(val)
    if str(val.type) == 'CSG_Array_Pointer': return array_pointer_printer(val)
    if str(val.type) == 'CSG_Parameter': return parameter_printer(val)
    #if str(val.type) == 'CSG_Parameters': return parameters_printer(val)
    if str(val.type) == 'CSG_String': return string_printer(val)
    if str(val.type) == 'CSG_String*': return string_printer(val)
    if str(val.type) == 'CSG_Strings': return strings_printer(val)
    if str(val.type) == 'CSG_Data_Object': return data_object_printer(val)
    if str(val.type) == 'CSG_Table': return table_printer(val)
    if str(val.type) == 'CSG_Projection': return projection_printer(val)
    if str(val.type) == 'CSG_Grid': return grid_printer(val)
    if str(val.type) == 'CSG_Grid_System': return grid_system_printer(val)
    if str(val.type) == 'CSG_MetaData': return meta_data_printer(val)
    if str(val.type) == 'CSG_DateTime': return datetime_printer(val)
    if str(val.type) == 'CSG_Rect': return rect_printer(val)
    if str(val.type) == 'CSG_Table_Record': return table_record_printer(val)
    if str(val.type) == 'CSG_Table_Record *': return table_record_printer(val)
    if str(val.type) == 'CSG_Shape': return shape_printer(val)
    if str(val.type) == 'CSG_Shape *': return shape_printer(val)
    if str(val.type) == 'CSG_Shape_Point': return shape_point_printer(val)
    if str(val.type) == 'CSG_Shape_Point *': return shape_point_printer(val)
    if str(val.type) == 'CSG_Shape_Points': return shape_points_printer(val)
    if str(val.type) == 'CSG_Shape_Points *': return shape_points_printer(val)
    if str(val.type) == 'CSG_Shape_Point_Z': return shape_point_z_printer(val)
    if str(val.type) == 'CSG_Shape_Point_Z *': return shape_point_z_printer(val)
    if str(val.type) == 'CSG_Shape_Point_ZM': return shape_point_zm_printer(val)
    if str(val.type) == 'CSG_Shape_Point_ZM *': return shape_point_zm_printer(val)
    if str(val.type) == 'CSG_Shapes *': return shapes_printer(val)
    if str(val.type) == 'CSG_Shapes': return shapes_printer(val)
    if str(val.type) == 'CSG_Simple_Statistics': return simple_statistics_printer(val)
    if str(val.type) == 'CSG_Simple_Statistics &': return simple_statistics_printer(val)

gdb.pretty_printers.append(my_pp_func)
